#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
*****************************************************************************************************
LIRIS Evaluator of human activity recognition and localisation algorithms - harl
Module "core" : package of function and objects without display

usage of this module :
- you just need to create an instance of the class Evaluator
- give it in constructor the file sets and threshold (mode impact the format of threshold)
- call the Process() method
- errors will raise harlExceptions
- comment will be printed on stderr

Authors : Tristan Delizy

ChangeLog :
01/06/2012 td: starting test and intense engine behaviour degug
24/06/2012 td: implementation of "basic" metrics (no temporal / space detection)
21/06/2012 td: finishing module "canexp" : takes a tkinter canvas and export it in png/pdf
20/06/2012 td: bugs fixed in export function, added the text one.
18/06/2012 td: export functions (.png and .pdf) - GUI
13/06/2012 td: adding user interface functions - GUI
07/06/2012 td: implementation of metrics v2 (temporal and spacial distinction for thresholds) - CORE - GUI
04/06/2012 td: working on core with new mode "gui"  - CORE
23/05/2012 - 01/06/2012 td: woking on GUI with tkinter - GUI
23/05/2012 td: begin GUI development. - GUI
21/05/2012 td: recast of the command line interface with getopt and output format. - CMD - CORE
16/05/2012 td: added crashing results in textfiles for plot mode - CORE
15/05/2012 td: added core functions to use "plot" mode in command line  - CMD - CORE
15/05/2012 td: addapted core to accept set of xml files instead of one file - CORE
15/05/2012 td: added the interface for command line (harl) - CMD
14/05/2012 td: reorganisation of code structure for more reusability (harl -> harlcore + (harl/harlgui) - CMD - CORE
11/05/2012 td: formulae implementation (recall, precision, FScore)
10/05/2012 td: xml parser working - add video name consistency check
07/05/2012 td: begin development.

List of knowns bugs :

Todo list :
transform the mordor into readable code :/

*****************************************************************************************************
"""
"""
*****************************************************************************************************
information for use :

XML files :
this application will use xml files structured in the following way :

<?xml version="1.0" encoding="UTF-8"?>
<tagset>
  <video>
    <videoName>VIDEO_1</videoName>
    <action nr="1" class="6">
      <bbox x="241" y="72" width="143" height="404" framenr="70"/>
      <bbox x="244" y="74" width="143" height="404" framenr="71"/>
      <bbox x="247" y="76" width="143" height="404" framenr="72"/>
      <bbox x="249" y="76" width="143" height="404" framenr="73"/>
      <bbox x="254" y="79" width="143" height="404" framenr="74"/>
      ...
    </action>
  </video>
  <video>
    <videoName>VIDEO_2</videoName>
    <action nr="2" class="1">
      <bbox x="100" y="72" width="100" height="350" framenr="50"/>
      <bbox x="101" y="74" width="100" height="375" framenr="51"/>
      <bbox x="102" y="76" width="102" height="390" framenr="52"/>
      <bbox x="103" y="78" width="104" height="400" framenr="53"/>
      <bbox x="104" y="80" width="102" height="400" framenr="54"/>
      ...
    </action>
  </video>
</tagset>

the file must have one or more <video> tag and for each one <videoName> tag.
A <video> contain 1..n <action> and an <action> 1..n <bbox>
The parser will not allow discontinuous and/or repetitives "framenr" attributes for the tag <bbox>
and check the presence of attribute.

the comparison is made on videos whith correspondant names in the two EvDataSet

*****************************************************************************************************
"""

#====================================================================================================#
#IMPORTS
#====================================================================================================#
import sys
import xml.etree.ElementTree as xml
import os
from time import localtime

#====================================================================================================#
#internal data structure and computing
#====================================================================================================#

#====================================================================================================HarlException
#class of exception to get differrent comportement for error treatments
class harlException(Exception):
    def __init__(self,cause):
        self.cause = cause

    def __str__(self):
        return self.cause

#names of activity class for recognition
classnames = ["discussion","give","box/desk","enter/leave","try-to-enter","unlock+enter","baggage","handshaking","typing","telephone"]

#----------------------------------------------------------------------------------------------------getAtt
#Return an attribute value from a node. Check for its existence
def getAtt(node,attname):
    rv=node.get(attname)
    if rv==None:
      e = "ERROR : parsing xml : attribute "+attname+" not found in tag "+node.tag
      raise harlException(e)
    return rv

#====================================================================================================Evaluator
#top class of the engine : contain both of the structures parsed from
#xml and will make the comparison and scoring.
class Evaluator:
  def __init__(self, mode, inputResultFiles="", groundTruthFiles="", trt=0, tpt=0, srt=0, spt=0, outdir="", fix=""):
    if mode == "perfvalue" and (inputResultFiles != "" and groundTruthFiles != "" and trt+tpt+srt+spt !=0):
      self.perfvalue_constructor(mode, inputResultFiles, groundTruthFiles, trt, tpt, srt, spt)
    elif mode == "plot" and (inputResultFiles != "" and groundTruthFiles != "" and trt+tpt+srt+spt !=0):
      self.plot_constructor(mode, inputResultFiles, groundTruthFiles, trt, tpt, srt, spt, outdir, fix)
    elif mode == "gui":
      self.gui_constructor(mode)
    elif mode == "basic" and inputResultFiles != "" and groundTruthFiles != "":
      self.basic_constructor(mode, inputResultFiles, groundTruthFiles)
    else:
      e = "ERROR : core failure - mode and/or options are not supported."
      raise harlException(e)

#----------------------------------------------------------------------------------------------------perfvalue_constructor
#Initialisation for perfvalue_mode
  def perfvalue_constructor(self,mode, inputResultFiles, groundTruthFiles, trt, tpt, srt, spt):
    #attributes
    self.mode = mode
    self.VideoToCompute = dict()
    self.BestMatching = dict()
    self.trt = trt						    #temporal recall threshold
    self.tpt = tpt						    #temporal precision threshold
    self.srt = srt						    #spatial recall threshold
    self.spt = spt						    #spatial precision threshold
    self.inputDataSet = 0					#input result object
    self.groundTruthDataSet = 0				#grund truth object

    #initialisation
    self.groundTruthDataSet = EvDataSet(groundTruthFiles)
    self.inputDataSet = EvDataSet(inputResultFiles)

#----------------------------------------------------------------------------------------------------plot_constructor
#Initialisation for plot_constructor
  def plot_constructor(self, mode, inputResultFiles, groundTruthFiles, trt, tpt, srt, spt, outdir, fix):
    #attributes
    self.mode = mode
    self.VideoToCompute = dict()
    self.BestMatching = dict()
    self.trt = trt						    #temporal recall threshold
    self.tpt = tpt						    #temporal precision threshold
    self.srt = srt						    #spatial recall threshold
    self.spt = spt						    #spatial precision threshold
    self.fix_trt = float(fix.split("_")[0])	#temporal recall fix value for precision variation computing
    self.fix_tpt = float(fix.split("_")[1])	#temporal precision fix value for recall variation computing
    self.fix_srt = float(fix.split("_")[2])	#spatial recall fix value for precision variation computing
    self.fix_spt = float(fix.split("_")[3])	#spatial precision fix value for recall variation computing
    self.inputDataSet = 0					#input result object
    self.groundTruthDataSet = 0				#grund truth object
    self.outdir = outdir					#directory name for results

    #default output directory
    if self.outdir == "":
      self.outdir = "output"

    #initialisation
    self.groundTruthDataSet = EvDataSet(groundTruthFiles)
    self.inputDataSet = EvDataSet(inputResultFiles)

#----------------------------------------------------------------------------------------------------gui_constructor
  #Initialisation for GUI_constructor
  def gui_constructor(self, mode):
    #attributes
    self.mode = mode
    self.VideoToCompute = dict()
    self.BestMatching = dict()
    self.fix_trt = 0						#temporal recall fix value for precision variation computing
    self.fix_tpt = 0						#temporal precision fix value for recall variation computing
    self.fix_srt = 0						#spatial recall fix value for precision variation computing
    self.fix_spt = 0						#spatial precision fix value for recall variation computing
    self.inputDataSet = 0					#input result object
    self.groundTruthDataSet = 0				#grund truth object
    self.outdir = "output"					#directory name for results

#----------------------------------------------------------------------------------------------------basic_constructor
#Initialisation for basic_constructor
  def basic_constructor(self, mode, inputResultFiles, groundTruthFiles):
    #attributes
    self.mode = mode
    self.VideoToCompute = dict()
    self.BestMatching = dict()
    self.inputDataSet = 0					#input result object
    self.groundTruthDataSet = 0					#grund truth object

    #initialisation
    self.groundTruthDataSet = EvDataSet("")
    self.inputDataSet = EvDataSet("")
    self.groundTruthDataSet.xmlFiles=groundTruthFiles
    self.inputDataSet.xmlFiles=inputResultFiles
    self.groundTruthDataSet.parseXml_basic()
    self.inputDataSet.parseXml_basic()

#----------------------------------------------------------------------------------------------------Process
#main data computing
  def Process(self):
    self.PreProcessing()
    self.RankFScore()
    sys.exit(1) #normal and clean out

#----------------------------------------------------------------------------------------------------GUI_Process
#data computing for gui application the threshold_var should be the name of the threshold in
#variation with samples count in this format : "name:samples"
#the threshold string should have this format : "tresholdName:fixValue:tresholdName:fixValue:..."
#extra or unknows thresholds names (in thresholdString) will be ignored, a missing threshold will raise a HarlException.
  def GUI_Process(self, inputResultFiles, groundTruthFiles, threshold_var, threshold_string):
    if inputResultFiles == "" or groundTruthFiles == "":
      e = "ERROR : core failure - empty files parameters in GUI_Process."
      raise harlException(e)

    self.groundTruthDataSet = None
    self.inputDataSet = None
    self.VideoToCompute.clear()

    self.groundTruthDataSet = EvDataSet(groundTruthFiles)
    self.inputDataSet = EvDataSet(inputResultFiles)
    self.PreProcessing()

    if threshold_var.split(":")[0] not in {"trt", "tpt", "srt", "spt"}:
      e = "ERROR : core failure - unknown threshold for variation in GUI_Process."
      raise harlException(e)

    try:
      self.fix_trt = float(threshold_string.split("trt")[1].split(":")[1])
      self.fix_tpt = float(threshold_string.split("tpt")[1].split(":")[1])
      self.fix_srt = float(threshold_string.split("srt")[1].split(":")[1])
      self.fix_spt = float(threshold_string.split("spt")[1].split(":")[1])
    except:
      e = "ERROR : core failure - unable to get the fix values in GUI_Process."
      raise harlException(e)

    result = self.RankFScore(threshold_var)
    return result

#----------------------------------------------------------------------------------------------------Confusion_Process
#
  def Confusion_Process(self, inputResultFiles, groundTruthFiles):

    if inputResultFiles == "" or groundTruthFiles == "":
      e = "ERROR : core failure - empty files parameters for datasets."
      raise harlException(e)

    self.groundTruthDataSet = None
    self.inputDataSet = None
    self.VideoToCompute.clear()

    self.groundTruthDataSet = EvDataSet(groundTruthFiles)
    self.inputDataSet = EvDataSet(inputResultFiles)

    dictD = self.inputDataSet.getVideos()
    dictG = self.groundTruthDataSet.getVideos()

    CMatrix = [[0 for col in range(10)] for row in range(10)]

    #define the set of video for computing
    for item in dictD:
      if dictG.has_key(item):
	self.VideoToCompute[item] = dictD[item]
    if len(self.VideoToCompute) == 0:
      e = "ERROR : no video matches between input data and ground truth data."
      raise harlException(e)
    for vid in self.VideoToCompute:
      self.BestMatching[vid] = EvBestMatchData(dictG[vid], dictD[vid], self, 1)

    for vid in self.VideoToCompute:
      for i in range(len(self.inputDataSet.getVideoByName(vid).getActions())):
	D_act = self.inputDataSet.getVideoByName(vid).getActions()[i]
	G_act = self.BestMatch(D_act, self.groundTruthDataSet.getVideoByName(vid).getActions(), vid)
	if isinstance(G_act, EvAction):
	  CMatrix[D_act.actionClass-1][G_act.actionClass-1] += 1

    return CMatrix


  def VideoComp_Process(self, video_Name):
    dictD = self.inputDataSet.getVideos()
    dictG = self.groundTruthDataSet.getVideos()
    result = []
    BestMatching = EvBestMatchData(dictG[video_Name], dictD[video_Name], self)
    for j in range(len(dictD[video_Name].getActions())):
	   bestmatch = self.BestMatch(dictD[video_Name].getActions()[j],dictG[video_Name].getActions(),video_Name, BestMatching)
	   if isinstance(bestmatch, EvAction):
            result.append([dictD[video_Name].getActions()[j],bestmatch])
    return result

#----------------------------------------------------------------------------------------------------PreProcessing
#checking that the videos in inputResultFile and groundTruthFile are the same
  def PreProcessing(self):
    dictD = self.inputDataSet.getVideos()
    dictG = self.groundTruthDataSet.getVideos()
    ignoredList = []

    #define the set of video for computing
    for item in dictD:
      if dictG.has_key(item):
        self.VideoToCompute[item] = dictD[item]
      else :
	ignoredList.append(dictD[item].getName())
    if len(self.VideoToCompute) == 0:
      e = "ERROR : no video matches between input data and ground truth data."
      raise harlException(e)
    if len(self.VideoToCompute) < len(dictD):
      e = "ERROR : these videos in the input dataset have no correspondense in ground truth:\n"
      for i in range(len(ignoredList)):
	e+=ignoredList[i]+" "
      raise harlException(e)
    if self.mode == "basic":
        return
    for vid in self.VideoToCompute:
      self.BestMatching[vid] = EvBestMatchData(dictG[vid], dictD[vid], self)

#----------------------------------------------------------------------------------------------------RankFScore
#ranking with the F-score (or harmonic mean)
  def RankFScore(self, threshold_var=""):
    print >> sys.stderr,"start computing"

    if self.mode == "perfvalue":
      pre = self.Precision_2(self.trt, self.tpt, self.srt, self.spt)
      rec = self.Recall_2(self.trt, self.tpt, self.srt, self.spt)
      if pre == 0 and rec == 0:
	   fsc = 0.0
      else:
	   fsc = float(2*pre*rec)/float(pre+rec)
      print >> sys.stderr, "\n\n"
      print rec, pre, fsc

    elif self.mode == "plot":
      trt_beg = self.trt[0]
      trt_end = self.trt[1]
      trt_step = self.trt[2]
      tpt_beg = self.tpt[0]
      tpt_end = self.tpt[1]
      tpt_step = self.tpt[2]
      srt_beg = self.srt[0]
      srt_end = self.srt[1]
      srt_step = self.srt[2]
      spt_beg = self.spt[0]
      spt_end = self.spt[1]
      spt_step = self.spt[2]

      rec_trt = []
      pre_trt = []
      fsc_trt = []

      rec_tpt = []
      pre_tpt = []
      fsc_tpt = []

      rec_srt = []
      pre_srt = []
      fsc_srt = []

      rec_spt = []
      pre_spt = []
      fsc_spt = []

      trt_wip = trt_beg
      tpt_wip = tpt_beg

      srt_wip = srt_beg
      spt_wip = spt_beg

      i=0
      while trt_wip < trt_end:
	   rec_trt.append(self.Recall_2(trt_wip, self.fix_tpt, self.fix_srt, self.fix_spt))
	   pre_trt.append(self.Precision_2(trt_wip, self.fix_tpt, self.fix_srt, self.fix_spt))
	   if rec_trt[i] != 0 and pre_trt[i] != 0:
	       fsc_trt.append(float(2*pre_trt[i]*rec_trt[i])/float(pre_trt[i]+rec_trt[i]))
	   else:
	       fsc_trt.append(0)
	   i += 1
	   trt_wip += trt_step

      i=0
      while tpt_wip < tpt_end:
	   rec_tpt.append(self.Recall_2(self.fix_trt, tpt_wip, self.fix_srt, self.fix_spt))
	   pre_tpt.append(self.Precision_2(self.fix_trt, tpt_wip, self.fix_srt, self.fix_spt))
	   if rec_tpt[i] != 0 and pre_tpt[i] != 0:
	       fsc_tpt.append(float(2*pre_tpt[i]*rec_tpt[i])/float(pre_tpt[i]+rec_tpt[i]))
	   else:
	       fsc_tpt.append(0)
	   i += 1
	   tpt_wip += tpt_step

      i=0
      while srt_wip < srt_end:
	   rec_srt.append(self.Recall_2(self.fix_trt, self.fix_tpt, srt_wip, self.fix_spt))
	   pre_srt.append(self.Precision_2(self.fix_trt, self.fix_tpt, srt_wip, self.fix_spt))
	   if rec_srt[i] != 0 and pre_srt[i] != 0:
	        fsc_srt.append(float(2*pre_srt[i]*rec_srt[i])/float(pre_srt[i]+rec_srt[i]))
	   else:
	    fsc_srt.append(0)
	   i += 1
	   srt_wip += srt_step

      i=0
      while spt_wip < spt_end:
	   rec_spt.append(self.Recall_2(self.fix_trt, self.fix_tpt, self.fix_srt, spt_wip))
	   pre_spt.append(self.Precision_2(self.fix_trt, self.fix_tpt, self.fix_srt, spt_wip))
	   if rec_spt[i] != 0 and pre_spt[i] != 0:
	       fsc_spt.append(float(2*pre_spt[i]*rec_spt[i])/float(pre_spt[i]+rec_spt[i]))
	   else:
	       fsc_spt.append(0)
	   i += 1
	   spt_wip += spt_step

      var_trt = [rec_trt, pre_trt, fsc_trt]
      var_tpt = [rec_tpt, pre_tpt, fsc_tpt]
      var_srt = [rec_srt, pre_srt, fsc_srt]
      var_spt = [rec_spt, pre_spt, fsc_spt]

      result_fscore = [var_trt[2],var_tpt[2],var_srt[2], var_spt[2]]

      result_FscAvg = []
      PV_Recall = round(self.Recall_2(self.fix_trt, self.fix_tpt, self.fix_srt, self.fix_spt),3)
      PV_Precision = round(self.Precision_2(self.fix_trt, self.fix_tpt, self.fix_srt, self.fix_spt),3)
      if PV_Recall != 0 and PV_Precision != 0:
	PV_Fsc = round(float(2*PV_Precision*PV_Recall)/float(PV_Precision+PV_Recall),3)
      else:
	PV_Fsc = 0.0

      for i in range(4):
	temp = 0
	for j in range(len(result_fscore[i])):
	  temp += result_fscore[i][j]
	temp = round(temp/float(len(result_fscore[i])),4)
	result_FscAvg.append(temp)

      integrated = (result_FscAvg[0]+result_FscAvg[1]+result_FscAvg[2]+result_FscAvg[3])/4.0

      print PV_Recall, PV_Precision, PV_Fsc, result_FscAvg[0], result_FscAvg[1], result_FscAvg[2], result_FscAvg[3], round(integrated,3)

      self.CrashToTextFile(var_trt, var_tpt, var_srt, var_spt)

    elif self.mode == "gui" and threshold_var != "":
      threshold_samples = int(threshold_var.split(":")[1])
      result_recall=[[],[],[],[]]
      result_precision=[[],[],[],[]]
      result_fscore=[[],[],[],[]]
      result_PerfVal = []
      result_FscAvg = []
      for i in range(threshold_samples+1):
	result_recall[0].append(self.Recall_2(float(i)/float(threshold_samples), self.fix_tpt, self.fix_srt, self.fix_spt))
	result_precision[0].append(self.Precision_2(float(i)/float(threshold_samples), self.fix_tpt, self.fix_srt, self.fix_spt))
	if result_recall[0][i] != 0 and result_precision[0][i] != 0:
	  result_fscore[0].append(float(2*result_precision[0][i]*result_recall[0][i])/float(result_precision[0][i]+result_recall[0][i]))
	else:
	  result_fscore[0].append(0.0)
      for i in range(threshold_samples+1):
	result_recall[1].append(self.Recall_2( self.fix_trt, float(i)/float(threshold_samples), self.fix_srt, self.fix_spt))
	result_precision[1].append(self.Precision_2( self.fix_trt, float(i)/float(threshold_samples), self.fix_srt, self.fix_spt))
	if result_recall[1][i] != 0 and result_precision[1][i] != 0:
	  result_fscore[1].append(float(2*result_precision[1][i]*result_recall[1][i])/float(result_precision[1][i]+result_recall[1][i]))
	else:
	  result_fscore[1].append(0.0)
      for i in range(threshold_samples+1):
	result_recall[2].append(self.Recall_2( self.fix_trt, self.fix_tpt,float(i)/float(threshold_samples), self.fix_spt))
	result_precision[2].append(self.Precision_2( self.fix_trt, self.fix_tpt, float(i)/float(threshold_samples), self.fix_spt))
	if result_recall[2][i] != 0 and result_precision[2][i] != 0:
	  result_fscore[2].append(float(2*result_precision[2][i]*result_recall[2][i])/float(result_precision[2][i]+result_recall[2][i]))
	else:
	  result_fscore[2].append(0.0)
      for i in range(threshold_samples+1):
	result_recall[3].append(self.Recall_2( self.fix_trt, self.fix_tpt, self.fix_srt, float(i)/float(threshold_samples)))
	result_precision[3].append(self.Precision_2( self.fix_trt, self.fix_tpt, self.fix_srt, float(i)/float(threshold_samples)))
	if result_recall[3][i] != 0 and result_precision[3][i] != 0:
	  result_fscore[3].append(float(2*result_precision[3][i]*result_recall[3][i])/float(result_precision[3][i]+result_recall[3][i]))
	else:
	  result_fscore[3].append(0.0)

      PV_Recall = round(self.Recall_2(self.fix_trt, self.fix_tpt, self.fix_srt, self.fix_spt),3)
      PV_Precision = round(self.Precision_2(self.fix_trt, self.fix_tpt, self.fix_srt, self.fix_spt),3)
      if PV_Recall != 0 and PV_Precision != 0:
	PV_Fsc = round(float(2*PV_Precision*PV_Recall)/float(PV_Precision+PV_Recall),3)
      else:
	PV_Fsc = 0.0
      result_PerfVal = [PV_Recall,PV_Precision, PV_Fsc]

      for i in range(4):
	temp = 0
	for j in range(len(result_fscore[i])):
	  temp += result_fscore[i][j]
	temp = round(temp/float(len(result_fscore[i])),3)
	result_FscAvg.append(temp)

      r = 0
      for key in ["trt","tpt","srt","spt"]:
	if threshold_var.split(":")[0] == key:
	  return [[result_recall[r], result_precision[r], result_fscore[r]],result_PerfVal,result_FscAvg]
	r += 1

    elif self.mode == "basic":
      pr = self.Precision_basic()
      re = self.Recall_basic()
      if pr == 0 and re == 0:
	fsc = 0.0
      else:
	fsc = float(2*pr*re)/float(pr+re)
	print >> sys.stderr, "\n\n"
	print re, pr, fsc
    else:
      e = "ERROR : core failure in RankFScore."
      raise harlException(e)


#----------------------------------------------------------------------------------------------------CrashToTextFile
#saving thr plot mode curves in output directories
  def CrashToTextFile(self, var_trt, var_tpt, var_srt, var_spt):
    if self.outdir == "output":
      outDir = os.getcwd()+"/"+self.outdir+str(localtime()[0])+"-"+str(localtime()[1])+"-"+str(localtime()[2])+"_"+str(localtime()[3])+"h"+str(localtime()[4])+"m"+str(localtime()[5])
    else:
      outDir = os.getcwd()+"/"+self.outdir

    os.makedirs(outDir+"/temporal_recall_variation")
    os.makedirs(outDir+"/temporal_precision_variation")
    os.makedirs(outDir+"/spatial_recall_variation")
    os.makedirs(outDir+"/spatial_precision_variation")

    f_trt_rec = open(outDir+"/temporal_recall_variation/recall_curve.txt", 'w')
    f_trt_pre = open(outDir+"/temporal_recall_variation/precision_curve.txt", 'w')
    f_trt_fsc = open(outDir+"/temporal_recall_variation/fscore_curve.txt", 'w')

    f_tpt_rec = open(outDir+"/temporal_precision_variation/recall_curve.txt", 'w')
    f_tpt_pre = open(outDir+"/temporal_precision_variation/precision_curve.txt", 'w')
    f_tpt_fsc = open(outDir+"/temporal_precision_variation/fscore_curve.txt", 'w')

    f_srt_rec = open(outDir+"/spatial_recall_variation/recall_curve.txt", 'w')
    f_srt_pre = open(outDir+"/spatial_recall_variation/precision_curve.txt", 'w')
    f_srt_fsc = open(outDir+"/spatial_recall_variation/fscore_curve.txt", 'w')

    f_spt_rec = open(outDir+"/spatial_precision_variation/recall_curve.txt", 'w')
    f_spt_pre = open(outDir+"/spatial_precision_variation/precision_curve.txt", 'w')
    f_spt_fsc = open(outDir+"/spatial_precision_variation/fscore_curve.txt", 'w')

    #writing files for trt variation
    for i in range(len(var_trt[0])):
      f_trt_rec.write(str(var_trt[0][i])+"\n")
    for i in range(len(var_trt[1])):
      f_trt_pre.write(str(var_trt[1][i])+"\n")
    for i in range(len(var_trt[2])):
      f_trt_fsc.write(str(var_trt[2][i])+"\n")

    #writing files for tpt variation
    for i in range(len(var_tpt[0])):
      f_tpt_rec.write(str(var_tpt[0][i])+"\n")
    for i in range(len(var_tpt[1])):
      f_tpt_pre.write(str(var_tpt[1][i])+"\n")
    for i in range(len(var_tpt[2])):
      f_tpt_fsc.write(str(var_tpt[2][i])+"\n")

    #writing files for srt variation
    for i in range(len(var_srt[0])):
      f_srt_rec.write(str(var_srt[0][i])+"\n")
    for i in range(len(var_srt[1])):
      f_srt_pre.write(str(var_srt[1][i])+"\n")
    for i in range(len(var_srt[2])):
      f_srt_fsc.write(str(var_srt[2][i])+"\n")

    #writing files for spt variation
    for i in range(len(var_spt[0])):
      f_spt_rec.write(str(var_spt[0][i])+"\n")
    for i in range(len(var_spt[1])):
      f_spt_pre.write(str(var_spt[1][i])+"\n")
    for i in range(len(var_spt[2])):
      f_spt_fsc.write(str(var_spt[2][i])+"\n")

    outDir = "./"+self.outdir
    print >> sys.stderr, "Results crashed in files : "
    print >> sys.stderr, "recall variation : \n\t"+outDir+"/recall_variation/recall_curve.txt\n\t"+outDir+"/recall_variation/precision_curve.txt\n\t"+outDir+"/recall_variation/fscore_curve.txt"
    print >> sys.stderr, "precision variation : \n\t"+outDir+"/precision_variation/recall_curve.txt\n\t"+outDir+"/precision_variation/precision_curve.txt\n\t"+outDir+"/precision_variation/fscore_curve.txt"


    f_trt_rec.close()
    f_trt_pre.close()
    f_trt_fsc.close()

    f_tpt_rec.close()
    f_tpt_pre.close()
    f_tpt_fsc.close()

    f_srt_rec.close()
    f_srt_pre.close()
    f_srt_fsc.close()

    f_spt_rec.close()
    f_spt_pre.close()
    f_spt_fsc.close()

#----------------------------------------------------------------------------------------------------BestMatch
#best match takes one action (actionToMatch) and compare it withe the
#list of action given in second parameter. Retrun an action from the
#parameter actions tab
  def BestMatch(self,actionToMatch, actions, videoToMatch = "", bestmatchingMatrix = None):
    if videoToMatch == "":
      bestMatch = 0
      bestValue = 0
      temp=0
      for i in range(len(actions)):
	if actionToMatch.getArea()==0 or actions[i].getArea() == 0:
	  e = "ERROR : one or more action have empty area."
	  raise harlException(e)
	else:
	  temp=2*float(self.ActionsOverlap(actionToMatch,actions[i]))/(actionToMatch.getArea() + actions[i].getArea())
	  if temp > bestValue:
	    bestMatch = i
	    bestValue = temp
      return actions[bestMatch]
    else:
      i = 0
      if bestmatchingMatrix == None:
        bm = self.BestMatching[videoToMatch]
      else:
        bm = bestmatchingMatrix
      bestMatch = 0
      bestValue = 0
      temp=0
      if actionToMatch in self.groundTruthDataSet.getVideoByName(videoToMatch).actions :
	vid = self.groundTruthDataSet.getVideoByName(videoToMatch)
	rank = vid.actions.index(actionToMatch)
	for i in range(len(bm.DtTab)):
	  if bm.Matrix[rank][i] > bestValue and bm.DtTab[i] == 1 :
	    bestMatch = i
	    bestValue = bm.Matrix[rank][i]
	if bestValue != 0:
	  bm.DtTab[bestMatch]=0
	  return actions[bestMatch]
	else:
	  return -1
      elif actionToMatch in self.inputDataSet.getVideoByName(videoToMatch).actions :
	vid = self.inputDataSet.getVideoByName(videoToMatch)
	rank = vid.actions.index(actionToMatch)
	for i in range(len(bm.GtTab)):
	  if bm.Matrix[i][rank] > bestValue and bm.GtTab[i] == 1 :
	    bestMatch = i
	    bestValue = bm.Matrix[i][rank]
	if bestValue != 0:
	  bm.GtTab[bestMatch]=0
	  return actions[bestMatch]
	else:
	  return -1
      else:
	e = "ERROR : core failure in BestMatch."
	raise harlException(e)
      return -1

#----------------------------------------------------------------------------------------------------IsMatched
#IsMatched tell with thresholds on area recall and precision if the 2
#action match or not return 0 or 1
  def IsMatched(self, tr, tp, matchingTarget, actionToMatch):
    if (matchingTarget.actionClass != actionToMatch.actionClass):
      return 0
    a = float(self.ActionsOverlap(matchingTarget, actionToMatch))/matchingTarget.getArea()
    b = float(self.ActionsOverlap(matchingTarget, actionToMatch))/actionToMatch.getArea()
    if (a > tr) and (b > tp):
      return 1
    else:
      return 0
#----------------------------------------------------------------------------------------------------IsMatched_2
#implementation in metric_2
  def IsMatched_2(self, trt, tpt, srt, spt, matchingTarget, actionToMatch):
    if matchingTarget.actionClass != actionToMatch.actionClass:
      return 0
    T_FramesOverlap = self.T_ActionOverlap(matchingTarget, actionToMatch)
    if matchingTarget.getFramesCount() != 0:
      t1 = float(float(T_FramesOverlap[2])/matchingTarget.getFramesCount())
    else :
      return 0
    if actionToMatch.getFramesCount()!= 0:
      t2 = float(float(T_FramesOverlap[2])/actionToMatch.getFramesCount())
    else :
      return 0
    if (t1 < trt) or (t2 < tpt):
      return 0
    if matchingTarget.getArea(str(T_FramesOverlap[0])+":"+str(T_FramesOverlap[1])) != 0:
      t3 = float(self.ActionsOverlap(matchingTarget, actionToMatch))/float(matchingTarget.getArea(str(T_FramesOverlap[0])+":"+str(T_FramesOverlap[1])))
    else :
      return 0
    if actionToMatch.getArea(str(T_FramesOverlap[0])+":"+str(T_FramesOverlap[1])) != 0:
      t4 = float(self.ActionsOverlap(matchingTarget, actionToMatch))/actionToMatch.getArea(str(T_FramesOverlap[0])+":"+str(T_FramesOverlap[1]))
    else :
      return 0
    if (t3 < srt) or (t4 < spt):
      return 0
    else:
      return 1
#----------------------------------------------------------------------------------------------------IsMatched_basic
#implementation for basic detection
  def IsMatched_basic(self, matchingTarget, actionToMatch):
    if (matchingTarget.actionClass != actionToMatch.actionClass):
      return 0
    else:
      return 1

#----------------------------------------------------------------------------------------------------Recall
#return the recall of the 2 dataset (focuses on the number of correctly
#detected actions on all action in the dataset) retrun a float
  def Recall(self, tr, tp):
    r1 = 0
    r2 = 0
    r = 0
    for item in self.VideoToCompute:
      r2 += len(self.groundTruthDataSet.getVideoByName(item).getActions())
      for j in range(len(self.groundTruthDataSet.getVideoByName(item).getActions())):
	r1 += self.IsMatched(tr, tp, self.groundTruthDataSet.getVideoByName(item).getActions()[j], self.BestMatch(self.groundTruthDataSet.getVideoByName(item).getActions()[j],self.inputDataSet.getVideoByName(item).getActions()))
    r=float(r1)/r2
    return r
#----------------------------------------------------------------------------------------------------Recall_2
#implementation in metric_2
  def Recall_2(self, trt, tpt, srt, spt):
    r1 = 0
    r2 = 0
    r = 0
    for item in self.VideoToCompute:
      r2 += len(self.groundTruthDataSet.getVideoByName(item).getActions())
      for j in range(len(self.groundTruthDataSet.getVideoByName(item).getActions())):
	bestmatch = self.BestMatch(self.groundTruthDataSet.getVideoByName(item).getActions()[j],self.inputDataSet.getVideoByName(item).getActions(),item)
	if isinstance(bestmatch, EvAction):
	  r1 += self.IsMatched_2(trt, tpt, srt, spt, self.groundTruthDataSet.getVideoByName(item).getActions()[j], bestmatch)
      self.BestMatching[item].reset_GtTab()
    if r2 != 0:
        r=float(r1)/r2
        return r
    else :
        return 0
#----------------------------------------------------------------------------------------------------Recall_basic
#implementation for basic detection
  def Recall_basic(self):
    r1 = 0
    r2 = 0
    r = 0
    matched = set()
    for item in self.VideoToCompute:
      matched.clear()
      matchingAct =  self.inputDataSet.getVideoByName(item).getActions()
      toMatchAct = self.groundTruthDataSet.getVideoByName(item).getActions()
      
      r2 += len(self.groundTruthDataSet.getVideoByName(item).getActions())
      for i in range(len(matchingAct)):
	j = 0
	test = 0
	while test == 0 and j < len(toMatchAct):
	  if toMatchAct[j] not in matched and self.IsMatched_basic(matchingAct[i], toMatchAct[j]) == 1:
	    r1 += 1
	    test = 1
	    matched.add(toMatchAct[j])
	  j += 1
	  
    #print "Number of groundtruth items:",r2
    #print "Number of overlap items",r1
    #print "Recall",float(r1)/r2
	  
    if r2 != 0:
        r=float(r1)/r2
        return r
    else :
        return 0

#----------------------------------------------------------------------------------------------------Precision
#return the precision of the 2 dataset (focuses on the number of wrong
#detection, taking into accout the total count of actions) retrun a float
  def Precision(self, tr, tp):
    r1 = 0
    r2 = 0
    r = 0
    for item in self.VideoToCompute:
      r2 += len(self.inputDataSet.getVideoByName(item).getActions())
      for j in range(len(self.inputDataSet.getVideoByName(item).getActions())):
	r1 += self.IsMatched(tr, tp, self.BestMatch(self.inputDataSet.getVideoByName(item).getActions()[j], self.groundTruthDataSet.getVideoByName(item).getActions()), self.inputDataSet.getVideoByName(item).getActions()[j])
    r=float(r1)/r2
    return r
#----------------------------------------------------------------------------------------------------Precision_2
#implementation in metric_2
  def Precision_2(self, trt, tpt, srt, spt):
    r1 = 0
    r2 = 0
    r = 0
    for item in self.VideoToCompute:
      r2 += len(self.inputDataSet.getVideoByName(item).getActions())
      for j in range(len(self.inputDataSet.getVideoByName(item).getActions())):
	bestmatch = self.BestMatch(self.inputDataSet.getVideoByName(item).getActions()[j], self.groundTruthDataSet.getVideoByName(item).getActions(), item)
	if isinstance(bestmatch, EvAction):
	  r1 += self.IsMatched_2(trt, tpt, srt, spt, bestmatch, self.inputDataSet.getVideoByName(item).getActions()[j])
      self.BestMatching[item].reset_DtTab()
    if r2 != 0:
        r=float(r1)/r2
        return r
    else :
        return 0
#----------------------------------------------------------------------------------------------------Precision_basic
#implementation for basic detection
  def Precision_basic(self):
    r1 = 0
    r2 = 0
    r = 0
    matched = set()
    
    # Iterate through videos
    for item in self.VideoToCompute:
      matched.clear()
      matchingAct =  self.groundTruthDataSet.getVideoByName(item).getActions()
      toMatchAct = self.inputDataSet.getVideoByName(item).getActions()
      test = 0
      r2 += len(self.inputDataSet.getVideoByName(item).getActions())
            
      # Iterate through the actions of the GT video
      for i in range(len(matchingAct)):
            
        # print item,matchingAct[i].actionClass,
      
        # Iterate through the actions of the result video
	j = 0
	test = 0 # cw
	while (test == 0) and (j < len(toMatchAct)):
	  if (toMatchAct[j] not in matched) and (self.IsMatched_basic(matchingAct[i], toMatchAct[j]) == 1):
	    r1 += 1
	    test = 1 # test == 1
	    matched.add(toMatchAct[j])
	    # print "yes"
	  j += 1
	#if test==0:
	#  print "no"
	  
    #print "Number of detected items:",r2
    #print "Number of overlap items",r1
    #print "Precision",float(r1)/r2
    	  
    if r2 != 0:
        r=float(r1)/r2
        return r
    else :
        return 0

#----------------------------------------------------------------------------------------------------FramesOverlap
#overlap calculation for 2 frames
  def FramesOverlap(self, rectA, rectB):
    rx = 0
    ry = 0
    r = 0
    rx = min(rectA.x2, rectB.x2) - max(rectA.x1,rectB.x1) 	#x1 < x2 see class EvRect
    if rx < 0 :
      rx = 0
    ry = min(rectA.y2, rectB.y2) - max(rectA.y1,rectB.y1)	#y1 < y2 see class EvRect
    if ry < 0 :
      ry = 0
    r = rx * ry
    return r

#----------------------------------------------------------------------------------------------------ActionsOverlap
#overlap calculation for 2 actions
  def ActionsOverlap(self, actionA, actionB):
    offsetA = 0
    offsetB = 0
    r = 0
    if actionA.getRect()[0].actId > actionB.getRect()[len(actionB.getRect())-1].actId:
      return 0
    if actionA.getRect()[len(actionA.getRect())-1].actId < actionB.getRect()[0].actId:
      return 0
    #ignoring frames with no correspondense in the other action
    while offsetA<(len(actionA.getRect())-1) and actionA.getRect()[offsetA].actId < actionB.getRect()[0].actId:
      offsetA+=1
    while offsetB<(len(actionB.getRect())-1) and actionB.getRect()[offsetB].actId < actionA.getRect()[0].actId:
      offsetB+=1
    while offsetA<len(actionA.getRect()) and offsetB<len(actionB.getRect()):
      r+=self.FramesOverlap(actionA.getRect()[offsetA], actionB.getRect()[offsetB])
      offsetA+=1
      offsetB+=1

    return r

#----------------------------------------------------------------------------------------------------T_ActionOverlap
#overlap calculation for 2 actions
  def T_ActionOverlap(self, actionA, actionB):
    offsetA = 0
    offsetB = 0
    overlap=0
    if actionA.getRect()[0].actId > actionB.getRect()[len(actionB.getRect())-1].actId:
      return [0,0,0]
    if actionA.getRect()[len(actionA.getRect())-1].actId < actionB.getRect()[0].actId:
      return [0,0,0]
    while offsetA<(len(actionA.getRect())-1) and actionA.getRect()[offsetA].actId < actionB.getRect()[0].actId:
      offsetA+=1
    while offsetB<(len(actionB.getRect())-1) and actionB.getRect()[offsetB].actId < actionA.getRect()[0].actId:
      offsetB+=1

    while (overlap+offsetA)<(len(actionA.getRect())-1) and (overlap+offsetB)<(len(actionB.getRect())-1):
      overlap += 1

    begin = actionA.getRect()[offsetA].actId
    end = actionA.getRect()[offsetA+overlap].actId
    count = 1+end - begin

    return [begin, end, count]

#====================================================================================================EvDataset
#Class working on one structure : make parsing and gives information for
#calculation
class EvDataSet:
  def __init__(self, xmlInputs=""):
    #attributes
    self.xmlFiles = xmlInputs					#path of the xml file
    self.Videos = dict()					#link video and names

    #initialisation
    if self.xmlFiles != "":
      self.parseXml()

#----------------------------------------------------------------------------------------------------getFile
#return the path of xml file bounded to the object
  def getFiles(self):
    return self.xmlFiles

#----------------------------------------------------------------------------------------------------getVideos
#return an array of evVideo
  def getVideos(self):
    return self.Videos

#----------------------------------------------------------------------------------------------------getVideoByName
#return an array of evVideo
  def getVideoByName(self, key):
    if self.Videos.has_key(key):
      return self.Videos[key]
    else:
      e = "ERROR : trying to access a video that doesn't seem to exist("+str(key)+")."
      raise harlException(e)

#----------------------------------------------------------------------------------------------------parseXml
#parsing xml and checking for validity of file (details at l.25)
  def parseXml(self):

    for f in self.xmlFiles:
      print >> sys.stderr, "parsing Xml file ("+f+")"
      try:
	tree = xml.parse(f)
      except:
	e = "ERROR : "+f+" cannot be read, check path and try again."
	raise harlException(e)

      rootElement = tree.getroot()

      # Get the video tags
      vids=tree.findall("video")
      if len(vids)<1:
	e = "ERROR : file "+f+" have no <video> tag."
	raise harlException(e)
      else :
	print >> sys.stderr, len(vids), "video found, loading video..."
	for i in range(len(vids)):

	  #Get the name of each video
	  names = vids[i].findall("videoName")
	  if len(names) > 1:
	    e = "ERROR : file "+f+" seems corrupted more than one <videoName> tag for one video."
	    raise harlException(e)
	  elif len(names) == 0:
	    e = "ERROR : video "+str(i)+" from file "+f+" have no name."
	    raise harlException(e)
	  #we don't need double
	  if names[0].text not in self.Videos:
	    print >> sys.stderr, "    video "+str(i)+" start loading with name "+names[0].text
	    self.Videos[names[0].text] = EvVideo(names[0].text)

	    #Get the actions in each video
	    actions = vids[i].findall("action")
	    if len(actions) < 1:
	      print >> sys.stderr, "WARNING            file "+f+" in video "+str(i)+" : no actions detected."
	      #e = "ERROR :  file "+f+" in video "+str(i)+" : no actions detected."
	      #raise harlException(e)
	    else:
	      print >> sys.stderr, "        video", i, ":", len(actions), "actions"
	    for j in range(len(actions)):
	      nr=int(getAtt(actions[j],"nr"))
	      cl=int(getAtt(actions[j],"class"))
	      act=EvAction(nr, cl)

	      #get the bounding boxes for each action
	      boxes=actions[j].findall("bbox")
	      if len(boxes) < 1:
		print >> sys.stderr, "WARNING            action",j,":", "no bounding boxes detected."
		box=EvRect(0,0,0,0,0)
		act.AddRect(box)
		#e = "ERROR :  file "+f+" in video "+str(i)+" , action "+str(j)+" : no bounding boxes detected."
		#raise harlException(e)
	      else:
		print >> sys.stderr, "            action",j,":",len(boxes), "boundig boxes"
		framesAdded = set()
		previousAdded = -1
		for k in range(len(boxes)):
		  idFrame=int(getAtt(boxes[k],"framenr"))
		  x=int(getAtt(boxes[k],"x"))
		  y=int(getAtt(boxes[k],"y"))
		  w=int(getAtt(boxes[k],"width"))
		  h=int(getAtt(boxes[k],"height"))
		  box=EvRect(x,y,x+w,y+h,idFrame)
		  box.Show()

		  #check for continuity and no repetition in the frames numérotation
		  if(idFrame in framesAdded or ((previousAdded != (idFrame-1) )and(previousAdded != -1))):
		    e = "ERROR :  file "+f+" in video "+str(i)+", action "+str(j)+" : bad frame numbering. prev= "+str(previousAdded)+" actual= "+str(idFrame)+" set= "+str(framesAdded)
		    raise harlException(e)
		  framesAdded.add(idFrame)
		  previousAdded = idFrame
		  act.AddRect(box)
	      print >> sys.stderr, "........................done"
	      self.Videos[names[0].text].AddAction(act)
	  else:
	    e = "ERROR : multiples instanciation on one video."
	    raise harlException(e)

#----------------------------------------------------------------------------------------------------parseXml_basic
#parsing xml and checking for validity of file for basic detection test
  def parseXml_basic(self):
    for f in self.xmlFiles:
      print >> sys.stderr, "parsing Xml file ("+f+")"
      try:
	tree = xml.parse(f)
      except:
	e = "ERROR : "+f+" cannot be read, check path and try again."
	raise harlException(e)

      rootElement = tree.getroot()

      # Get the video tags
      vids=tree.findall("video")
      if len(vids)<1:
	e = "ERROR : file "+f+" have no <video> tag."
	raise harlException(e)
      else :
	print >> sys.stderr, len(vids), "video found, loading video..."
	for i in range(len(vids)):

	  #Get the name of each video
	  names = vids[i].findall("videoName")
	  if len(names) > 1:
	    e = "ERROR : file "+f+" seems corrupted more than one <videoName> tag for one video."
	    raise harlException(e)
	  elif len(names) == 0:
	    e = "ERROR : video "+str(i)+" from file "+f+" have no name."
	    raise harlException(e)
	  #we don't need double
	  if names[0].text not in self.Videos:
	    print >> sys.stderr, "    video "+str(i)+" start loading with name "+names[0].text
	    self.Videos[names[0].text] = EvVideo(names[0].text)

	    #Get the actions in each video
	    actions = vids[i].findall("action")
	    if len(actions) < 1:
	      print >> sys.stderr, "WARNING            file "+f+" in video "+str(i)+" : no actions detected."
	      #e = "ERROR :  file "+f+" in video "+str(i)+" : no actions detected."
	      #raise harlException(e)
	    else:
	      print >> sys.stderr, "        video", i, ":", len(actions), "actions"
	    for j in range(len(actions)):
	      nr=int(getAtt(actions[j],"nr"))
	      cl=int(getAtt(actions[j],"class"))
	      act=EvAction(nr, cl)

	      self.Videos[names[0].text].AddAction(act)
	  else:
	    e = "ERROR : multiples instanciation on one video."
	    raise harlException(e)

#====================================================================================================EvVideo
#class storing information from one video
class EvVideo:
  def __init__(self, inputName):
    #attributes
    self.name = ""						#video name (from xml)
    self.actions=[]						#array of actions

    #initialisation
    self.name = inputName

#----------------------------------------------------------------------------------------------------getActions
#return an array of actions
  def getActions(self):
    return self.actions

#----------------------------------------------------------------------------------------------------getName
  def getName(self):
    return self.name

#----------------------------------------------------------------------------------------------------AddAction
  def AddAction(self, inputAction):
    self.actions.append(inputAction)

#====================================================================================================EvAction
#class storing information from one action
class EvAction:
  def __init__(self, inputIdAction, inputActionClass):
    #attributes
    self.idAction = 0						#identification of this action
    self.actionClass = 0					#class of action detected
    self.rects=[]						#array of EvRect

    #initialisation
    self.idAction = inputIdAction
    self.actionClass = inputActionClass

#----------------------------------------------------------------------------------------------------getFramesCount
  def getFramesCount(self):
    return (len(self.rects))


#----------------------------------------------------------------------------------------------------getRect
  def getRect(self):
    return self.rects

#----------------------------------------------------------------------------------------------------getArea
#format of rangeString : "begin:end"
  def getArea(self, rangeString=""):
    r = 0
    if rangeString != "":
      try:
	actId_begin = int(rangeString.split(":")[0])
	actId_end = int(rangeString.split(":")[1])
      except:
	e = "ERROR : in EvVideo.getArea(), rangeString cannot be read."
	raise harlException(e)
      offset = actId_begin - self.rects[0].actId
      if offset < 0 or (len(self.rects)+self.rects[0].actId) < actId_end:
	return 0
      for i in range(actId_begin-self.rects[0].actId, actId_end-self.rects[0].actId+1):
	r += (self.rects[i].x2-self.rects[i].x1)*(self.rects[i].y2-self.rects[i].y1)
    else:
      for i in range(len(self.rects)):
	r += (self.rects[i].x2-self.rects[i].x1)*(self.rects[i].y2-self.rects[i].y1)
    return r

#----------------------------------------------------------------------------------------------------AddRect
  def AddRect(self, inputRect):
    self.rects.append(inputRect)

#----------------------------------------------------------------------------------------------------Show
  def Show(self):
    print >> sys.stderr, "action : id = ",self.idAction, "class = ", self.actionClass


#====================================================================================================EvRect
#class storing information from one frame in one action
class EvRect:
  def __init__(self, x1, y1, x2, y2, actId):
    #forcing the attributes to be ascending ordered
    if x1<x2:
        self.x1=x1
        self.x2=x2
    else:
        self.x1=x2
        self.x2=x1
    if y1<y2:
        self.y1=y1
        self.y2=y2
    else:
        self.y1=y2
        self.y2=y1
    self.actId=actId

#----------------------------------------------------------------------------------------------------Show
  def Show(self):
    print >> sys.stderr, "			x1=", self.x1, "  y1=", self.y1, "  x2=", self.x2, "  y2=", self.y2, "  id=", self.actId


#====================================================================================================EvBestMatchData
#class storing information computed for best match
class EvBestMatchData:
#----------------------------------------------------------------------------------------------------__init__
#constructor
  def __init__(self, Gvideo, Dvideo, Evaluator, confusion = 0):
    if not isinstance(Gvideo, EvVideo) or not isinstance(Dvideo, EvVideo):
      e = "ERROR : in EvBestMatchData.__init__, parameters 1 and 2 must be instances of EvVideo class."
      raise harlException(e)
    self.parent = Evaluator
    self.Gvid = Gvideo
    self.Dvid = Dvideo
    self.Matrix = [[0 for col in range(len(self.Dvid.actions))] for row in range(len(self.Gvid.actions))]
    self.GtTab = [1 for col in range(len(self.Gvid.actions))]
    self.DtTab = [1 for col in range(len(self.Dvid.actions))]

    if confusion == 0 :
      for i in range(len(self.Gvid.actions)) :
      	for j in range(len(self.Dvid.actions)):
	  if self.Gvid.actions[i].actionClass == self.Dvid.actions[j].actionClass:
	    self.Matrix[i][j] = float(2*self.parent.ActionsOverlap(self.Gvid.actions[i], self.Dvid.actions[j])) / (float(self.Gvid.actions[i].getArea())+float(self.Dvid.actions[j].getArea()))
	  else:
	    self.Matrix[i][j] = 0
    else:
      for i in range(len(self.Gvid.actions)) :
      	for j in range(len(self.Dvid.actions)):
	  self.Matrix[i][j] = float(2*self.parent.ActionsOverlap(self.Gvid.actions[i], self.Dvid.actions[j])) / (float(self.Gvid.actions[i].getArea())+float(self.Dvid.actions[j].getArea()))

#----------------------------------------------------------------------------------------------------reset_GtTab
#reset the bestmatching ground truth appariement array
  def reset_GtTab(self):
    self.GtTab = [1 for col in range(len(self.Gvid.actions))]

#----------------------------------------------------------------------------------------------------reset_DtTab
#reset the bestmatching detection appariement array
  def reset_DtTab(self):
    self.DtTab = [1 for col in range(len(self.Dvid.actions))]

